// ===================================================================
// UPS for Raspberry Pi
// Hack Addon for Geekworm Power Pack Pro V1.1 Lithium Battery Power Source UPS HAT 
// https://www.banggood.com/Geekworm-Power-Pack-Pro-V1_1-Lithium-Battery-Power-Source-UPS-HAT-Expansion-Board-For-Raspberry-Pi-p-1205973.html?rmmds=myorder&cur_warehouse=CN
// http://www.raspberrypiwiki.com/index.php/Power_Pack_Pro
// ===================================================================
// Author : JLC
// Date : 11/02/2018
// ===================================================================
// Operation :
// RaspberryPi can be powered either by USB-in or LiPo, UPS module toggles between two of them automatically. BUT DOES NOT manage STOP power states.
// This Hack Addon uses Arduino Nano to control UPS and thus power to the Pi : it acts as someone presses main button, shortly one click, shortly double click or one long click.
// Nano behaves like a i2c slave : RaspberryPi will send control commands through i2c bus (see existing command below).
// RaspberryPi must have an active daemon (see example script on end of file).

// Arduino Program operation :
// When nano is powered, it switches on module : RaspberryPi is on.
// Either Lipo safe delay is out and RaspberryPi is cut off,
// Or RaspberryPi sends i2c STOP command (as soon as it detects USB-In is absent) that triggers stop procedure (wait 30s (default) and siwtch off module and Pi)
// If USB-In came back before the procedure ends, RaspberryPi is powered off and on again to start a new booting stage.

// RaspberryPi daemon operation :
// Periodically, raspberry pi daemon requests USB-In presence through i2c bus. When there's no USB-In anymore, it sends STOP command and halt itself cleanly.

// ===================================================================
// i2c commands :
// - Master sends 0xFF to switch off module, can send 2nd value for specific timeout before starting STOP procedure (unit is 10s) - Those two values must be sent in the same request.
// - Master requests USB-in presence : 1 means USB-in plugged, 0 means USB-in unplugged
//
// ===================================================================

// ===================================================================
// Script example to place to Raspberry pi : UPS.sh
// File must have X right : sudo chmod +x UPS.sh
// File must be launched at beginning : add command at the end of file /etc/rc.local, just before exit 0
// command example : su -c "/home/pi/UPS/UPS.sh &" - pi
//
// i2c address is 0x64 (maybe changed, see code - be carreful, 0x62 is already used)
// STOP procedure is launched after 40s (i2cset -y 1 0x64 0xFF 4)
// ===================================================================
// #!/bin/sh
// 
// while [ true ]
// do
// 
//         state=$(i2cget -y 1 0x64)
//         echo $state
//         if [ $((state)) -eq 0 ]; then
//                 echo "UPS : STOPPING SYSTEM !"
//                 i2cset -y 1 0x64 0xFF 4
//                 sudo halt -p
//                 exit 0
//         fi
// 
//         # delay between 2 readings (seconds)
//         sleep 10
// 
// done




#include <Wire.h>
// ===================================================================
#define I2C_SLAVE_ADDRESS           0x64
#define DELAY_BEFORE_SWITCH_OFF_S   40
#define LIPO_SAFE_TIMEOUT_DELAY_S   300
// ===================================================================

// Global variable - used from interrupt code
bool bStop;                                   // True when i2c STOP command has been received.
unsigned long ulSecondDelayBeforeSwitchOff;   // Delay before stops when i2c STOP command has been received. Unit is 1s


// ===================================================================
// MAIN PROGRAM
void setup() {
  // IO setup
  pinMode( 5, INPUT );
  pinMode( 12, OUTPUT );

  // default values
  bStop = false;
  ulSecondDelayBeforeSwitchOff = DELAY_BEFORE_SWITCH_OFF_S;

  // i2c setup
  Wire.begin( I2C_SLAVE_ADDRESS );               // i2c slave listens to 0x64 address (I2C_SLAVE_ADDRESS)
  Wire.onReceive( receiveEvent ); // register event used to launch STOP procedure (2nd byte option sets a specific delay before starting procedure)
  Wire.onRequest( requestEvent ); // register event used to return USB-in presence

}

void loop() {

unsigned short usTimeOut;

  // Switch ON Pi
  doLongPress();

  // Wait for i2c stop command coming
  usTimeOut = LIPO_SAFE_TIMEOUT_DELAY_S;
  while( bStop == false )
  {
    // Switch off module if USB-in is unplugged during about 5 minutes
    if( digitalRead( 5 ) == 0 )
    {
      usTimeOut--;
      if ( usTimeOut == 0 )
      {
        // Switch off module
        doDoubleShortPress();

        // delay to be switched off safely (should be stopped now !!!)
        delay( 1000 );
      }
    }
    else
      usTimeOut = LIPO_SAFE_TIMEOUT_DELAY_S;

    // Delay 1s (must be 1s because of usTimeOut unit)
    delay( 1000 );
    
  }

  // Wait initially 40s (time raspberry pi is normally down) - maybe overwritten by i2c STOP command
  delay( ulSecondDelayBeforeSwitchOff * 1000 );

  // Switch off module
  doDoubleShortPress();

  // Delay for module to stop
  delay( 1000 );

  // If program is still running here, Power has come back => Raspberry has not been switched off !

  // Switch OFF Pi
  doLongPress();

  // Wait 
  delay( 1000 );

  // Reset State
  bStop = false;
 
}


// ===================================================================
// I2C CALL BACK FUNCTIONS

void receiveEvent( int numBytes ) {
unsigned char commandByte;
unsigned char ucDelay10s;

  if ( Wire.available() < 1 ) return;

  commandByte = Wire.read();

  // delayed switch off
  if ( commandByte == 0xFF )
  {

    // 2nd byte option : if present, gives delay to wait (unit is 10s)
    if ( Wire.available() != 0 )
    {
      
      ucDelay10s = Wire.read();
      ulSecondDelayBeforeSwitchOff = ucDelay10s * 10;  // Uses received delay before launching switch off procedure

    }

    // Launch STOP procedure
    bStop = true;
    
  }
  
}

void requestEvent( )
{
  if ( digitalRead( 5 ) != 0 )
    Wire.write( 1 );  // USB power In is plugged
   else
    Wire.write( 0 );  // USB power In is not plugged
}


// ===================================================================
// FUNCTIONS

void doLongPress( )
{
  // Simulate long press to switch on 
  digitalWrite( 13, HIGH );
  digitalWrite( 12, HIGH );

  delay( 3000 );

  digitalWrite( 13, LOW );
  digitalWrite( 12, LOW );

  delay( 3000 );
}

void doShortPress( )
{
  // Simulate long press to switch on 
  digitalWrite( 13, HIGH );
  digitalWrite( 12, HIGH );

  delay( 100 );

  digitalWrite( 13, LOW );
  digitalWrite( 12, LOW );

  delay( 100 );
}

void doDoubleShortPress()
{
  doShortPress();
  doShortPress();
  delay( 100 );
}

